function plot_persistence_diagram(persistence_threshold, dims, births, deaths)
    %% apply persistence_threshold
    thresholded_points = deaths - births > persistence_threshold;
    dims = dims( thresholded_points );
    births = births( thresholded_points );
    deaths = deaths( thresholded_points );

    %% classify points in diagram     
    essentials = (dims < 0);
    ordinaries = ~essentials;
    
    %% draw parameters
    max_dims = max(floor(abs(dims+0.5)));
    cur_colormap = lines( max_dims + 1 );
    colormap( cur_colormap );
    marker_size = 50;
    
    %% draw essential points
    scatter( births( essentials ), deaths( essentials ), marker_size, cur_colormap(-dims( essentials ) - 1 + 1,:), ...
             'filled', 'MarkerEdgeColor','k', 'Marker', 's' );
         
    %% draw non essential points
    hold on
    scatter( births( ordinaries ), deaths( ordinaries ), marker_size, cur_colormap(dims( ordinaries ) + 1,:), ...
             'filled', 'MarkerEdgeColor','k', 'Marker', 'o' );
    hold off
    
    %% draw diagonal
    min_value = min( births );
    max_value = max( deaths );
    line( [min_value, max_value], [min_value, max_value], 'Color','k' );

    %% draw axis
    safe_min = min_value - 10 * eps( min_value );
    safe_max = max_value + 10 * eps( max_value );
    axis square  
    axis( [safe_min, safe_max, safe_min, safe_max] );
    xticks = get(gca,'XTick');
    set(gca,'YTick',xticks);

    
    %% draw legend
    legend('Essential','Ordinary','Location','SouthEast');
    
    %% draw axis labels
    xlabel('Birth');
    ylabel('Death');

    %% draw colorbar

    if (isempty(max_dims))
        max_dims = 0;
    end
 %   disp(max_dims);
    caxis([0 max_dims]);
    ylabel(colorbar('YTick',0:1:max_dims), 'Dimension');
    
    %% draw grid
    grid on;
end
